"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndyLedgerService = void 0;
const tsyringe_1 = require("tsyringe");
const AgentConfig_1 = require("../../../agent/AgentConfig");
const AriesFrameworkError_1 = require("../../../error/AriesFrameworkError");
const IndySdkError_1 = require("../../../error/IndySdkError");
const indyError_1 = require("../../../utils/indyError");
const IndyWallet_1 = require("../../../wallet/IndyWallet");
const indy_1 = require("../../indy");
let IndyLedgerService = class IndyLedgerService {
    constructor(wallet, agentConfig, indyIssuer) {
        this.wallet = wallet;
        this.agentConfig = agentConfig;
        this.indy = agentConfig.agentDependencies.indy;
        this.logger = agentConfig.logger;
        this.indyIssuer = indyIssuer;
        this.fileSystem = agentConfig.fileSystem;
        // Listen to stop$ (shutdown) and close pool
        agentConfig.stop$.subscribe(async () => {
            if (this._poolHandle) {
                await this.close();
            }
        });
    }
    async getPoolHandle() {
        if (!this._poolHandle) {
            return this.connect();
        }
        return this._poolHandle;
    }
    async close() {
        // FIXME: Add type to indy-sdk
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        await this.indy.closePoolLedger(this._poolHandle);
        this._poolHandle = undefined;
    }
    async delete() {
        // Close the pool if currently open
        if (this._poolHandle) {
            await this.close();
        }
        // FIXME: Add type to indy-sdk
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        await this.indy.deletePoolLedgerConfig(this.agentConfig.poolName);
    }
    async connect() {
        const poolName = this.agentConfig.poolName;
        const genesisPath = await this.getGenesisPath();
        if (!genesisPath) {
            throw new Error('Cannot connect to ledger without genesis file');
        }
        this.logger.debug(`Connecting to ledger pool '${poolName}'`, { genesisPath });
        try {
            this.logger.debug(`Creating pool '${poolName}'`);
            await this.indy.createPoolLedgerConfig(poolName, { genesis_txn: genesisPath });
        }
        catch (error) {
            if (indyError_1.isIndyError(error, 'PoolLedgerConfigAlreadyExistsError')) {
                this.logger.debug(`Pool '${poolName}' already exists`, {
                    indyError: 'PoolLedgerConfigAlreadyExistsError',
                });
            }
            else {
                throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
            }
        }
        try {
            this.logger.debug('Setting ledger protocol version to 2');
            await this.indy.setProtocolVersion(2);
            this.logger.debug(`Opening pool ${poolName}`);
            this._poolHandle = await this.indy.openPoolLedger(poolName);
            return this._poolHandle;
        }
        catch (error) {
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async registerPublicDid(submitterDid, targetDid, verkey, alias, role) {
        try {
            this.logger.debug(`Register public did on ledger '${targetDid}'`);
            const request = await this.indy.buildNymRequest(submitterDid, targetDid, verkey, alias, role || null);
            const response = await this.submitWriteRequest(request, submitterDid);
            this.logger.debug(`Registered public did '${targetDid}' on ledger`, {
                response,
            });
            return targetDid;
        }
        catch (error) {
            this.logger.error(`Error registering public did '${targetDid}' on ledger`, {
                error,
                submitterDid,
                targetDid,
                verkey,
                alias,
                role,
                poolHandle: await this.getPoolHandle(),
            });
            throw error;
        }
    }
    async getPublicDid(did) {
        try {
            this.logger.debug(`Get public did '${did}' from ledger`);
            const request = await this.indy.buildGetNymRequest(null, did);
            this.logger.debug(`Submitting get did request for did '${did}' to ledger`);
            const response = await this.indy.submitRequest(await this.getPoolHandle(), request);
            const result = await this.indy.parseGetNymResponse(response);
            this.logger.debug(`Retrieved did '${did}' from ledger`, result);
            return result;
        }
        catch (error) {
            this.logger.error(`Error retrieving did '${did}' from ledger`, {
                error,
                did,
                poolHandle: await this.getPoolHandle(),
            });
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async registerSchema(did, schemaTemplate) {
        try {
            this.logger.debug(`Register schema on ledger with did '${did}'`, schemaTemplate);
            const { name, attributes, version } = schemaTemplate;
            const schema = await this.indyIssuer.createSchema({ originDid: did, name, version, attributes });
            const request = await this.indy.buildSchemaRequest(did, schema);
            const response = await this.submitWriteRequest(request, did);
            this.logger.debug(`Registered schema '${schema.id}' on ledger`, {
                response,
                schema,
            });
            schema.seqNo = response.result.txnMetadata.seqNo;
            return schema;
        }
        catch (error) {
            this.logger.error(`Error registering schema for did '${did}' on ledger:`, {
                error,
                did,
                poolHandle: await this.getPoolHandle(),
                schemaTemplate,
            });
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async getSchema(schemaId) {
        try {
            this.logger.debug(`Get schema '${schemaId}' from ledger`);
            const request = await this.indy.buildGetSchemaRequest(null, schemaId);
            this.logger.debug(`Submitting get schema request for schema '${schemaId}' to ledger`);
            const response = await this.submitReadRequest(request);
            const [, schema] = await this.indy.parseGetSchemaResponse(response);
            this.logger.debug(`Got schema '${schemaId}' from ledger`, {
                response,
                schema,
            });
            return schema;
        }
        catch (error) {
            this.logger.error(`Error retrieving schema '${schemaId}' from ledger`, {
                error,
                schemaId,
                poolHandle: await this.getPoolHandle(),
            });
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async registerCredentialDefinition(did, credentialDefinitionTemplate) {
        try {
            this.logger.debug(`Register credential definition on ledger with did '${did}'`, credentialDefinitionTemplate);
            const { schema, tag, signatureType, supportRevocation } = credentialDefinitionTemplate;
            const credentialDefinition = await this.indyIssuer.createCredentialDefinition({
                issuerDid: did,
                schema,
                tag,
                signatureType,
                supportRevocation,
            });
            const request = await this.indy.buildCredDefRequest(did, credentialDefinition);
            const response = await this.submitWriteRequest(request, did);
            this.logger.debug(`Registered credential definition '${credentialDefinition.id}' on ledger`, {
                response,
                credentialDefinition: credentialDefinition,
            });
            return credentialDefinition;
        }
        catch (error) {
            this.logger.error(`Error registering credential definition for schema '${credentialDefinitionTemplate.schema.id}' on ledger`, {
                error,
                did,
                poolHandle: await this.getPoolHandle(),
                credentialDefinitionTemplate,
            });
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async getCredentialDefinition(credentialDefinitionId) {
        try {
            this.logger.debug(`Get credential definition '${credentialDefinitionId}' from ledger`);
            const request = await this.indy.buildGetCredDefRequest(null, credentialDefinitionId);
            this.logger.debug(`Submitting get credential definition request for credential definition '${credentialDefinitionId}' to ledger`);
            const response = await this.submitReadRequest(request);
            const [, credentialDefinition] = await this.indy.parseGetCredDefResponse(response);
            this.logger.debug(`Got credential definition '${credentialDefinitionId}' from ledger`, {
                response,
                credentialDefinition,
            });
            return credentialDefinition;
        }
        catch (error) {
            this.logger.error(`Error retrieving credential definition '${credentialDefinitionId}' from ledger`, {
                error,
                credentialDefinitionId: credentialDefinitionId,
                poolHandle: await this.getPoolHandle(),
            });
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async submitWriteRequest(request, signDid) {
        try {
            const requestWithTaa = await this.appendTaa(request);
            const signedRequestWithTaa = await this.signRequest(signDid, requestWithTaa);
            const response = await this.indy.submitRequest(await this.getPoolHandle(), signedRequestWithTaa);
            if (response.op === 'REJECT') {
                throw new AriesFrameworkError_1.AriesFrameworkError(`Ledger rejected transaction request: ${response.reason}`);
            }
            return response;
        }
        catch (error) {
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async submitReadRequest(request) {
        try {
            const response = await this.indy.submitRequest(await this.getPoolHandle(), request);
            if (response.op === 'REJECT') {
                throw Error(`Ledger rejected transaction request: ${response.reason}`);
            }
            return response;
        }
        catch (error) {
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async signRequest(did, request) {
        try {
            return this.indy.signRequest(this.wallet.handle, did, request);
        }
        catch (error) {
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async appendTaa(request) {
        try {
            const authorAgreement = await this.getTransactionAuthorAgreement();
            // If ledger does not have TAA, we can just send request
            if (authorAgreement == null) {
                return request;
            }
            const requestWithTaa = await this.indy.appendTxnAuthorAgreementAcceptanceToRequest(request, authorAgreement.text, authorAgreement.version, authorAgreement.digest, this.getFirstAcceptanceMechanism(authorAgreement), 
            // Current time since epoch
            // We can't use ratification_ts, as it must be greater than 1499906902
            Math.floor(new Date().getTime() / 1000));
            return requestWithTaa;
        }
        catch (error) {
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    async getTransactionAuthorAgreement() {
        try {
            // TODO Replace this condition with memoization
            if (this.authorAgreement !== undefined) {
                return this.authorAgreement;
            }
            const taaRequest = await this.indy.buildGetTxnAuthorAgreementRequest(null);
            const taaResponse = await this.submitReadRequest(taaRequest);
            const acceptanceMechanismRequest = await this.indy.buildGetAcceptanceMechanismsRequest(null);
            const acceptanceMechanismResponse = await this.submitReadRequest(acceptanceMechanismRequest);
            // TAA can be null
            if (taaResponse.result.data == null) {
                this.authorAgreement = null;
                return null;
            }
            // If TAA is not null, we can be sure AcceptanceMechanisms is also not null
            const authorAgreement = taaResponse.result.data;
            const acceptanceMechanisms = acceptanceMechanismResponse.result.data;
            this.authorAgreement = Object.assign(Object.assign({}, authorAgreement), { acceptanceMechanisms });
            return this.authorAgreement;
        }
        catch (error) {
            throw indyError_1.isIndyError(error) ? new IndySdkError_1.IndySdkError(error) : error;
        }
    }
    getFirstAcceptanceMechanism(authorAgreement) {
        const [firstMechanism] = Object.keys(authorAgreement.acceptanceMechanisms.aml);
        return firstMechanism;
    }
    async getGenesisPath() {
        // If the path is already provided return it
        if (this.agentConfig.genesisPath)
            return this.agentConfig.genesisPath;
        // Determine the genesisPath
        const genesisPath = this.fileSystem.basePath + `/afj/genesis-${this.agentConfig.poolName}.txn`;
        // Store genesis data if provided
        if (this.agentConfig.genesisTransactions) {
            await this.fileSystem.write(genesisPath, this.agentConfig.genesisTransactions);
            return genesisPath;
        }
        // No genesisPath
        return null;
    }
};
IndyLedgerService = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [IndyWallet_1.IndyWallet, AgentConfig_1.AgentConfig, indy_1.IndyIssuerService])
], IndyLedgerService);
exports.IndyLedgerService = IndyLedgerService;
//# sourceMappingURL=IndyLedgerService.js.map